
/**
 * Returns $NCHOME
 */
function getNchome()
{
    var propMgr = Components.classes["@micromuse.com/NCIProperties;1"].getService( Components.interfaces.nsINCIProperties );

    return propMgr.GetProp( "NCHOME.INSTALLDIR" ).value;
}


/**
 * Read the contents of a file.
 *
 * @param filename (String) The full path of the file to read from.
 * @return (String) Contents of the file.
 */
function readFromFile( filename )
{
    // Read the file.
    log( 2, "Reading file: " + filename );

	// Create and initialise the file.
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
    // NOTE: This will not work with Mac!
    file.initWithPath( filename );

    // Use IOService to convert the file to a file URI.
    var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService( Components.interfaces.nsIIOService );
    var fileUri = ioService.newFileURI( file.QueryInterface( Components.interfaces.nsIFile ) );

    // Get a channel from the URI.
    var channel = ioService.newChannelFromURI( fileUri );

    // Create an input stream.
    var inStream = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance( Components.interfaces.nsIScriptableInputStream );

    var fileContents = "";

	try
    {
        inStream.init( channel.open() );

        var bytes;

        while ( (bytes = inStream.available()) > 0 )
        {
            fileContents += inStream.read( bytes );
        }
    }
    catch ( e )
    {
        log( 0, e );
    }
    finally
    {
        inStream.close();
    }

    log( 3, "Read completed." );

    return fileContents;
}


/**
 * Write content to a file. The file will have 664 permissions.
 *
 * @param filename (String) The full path of the file to read from.
 * @param content (String) The content to write.
 */
function writeToFile( filename, content )
{
    log( 2, "Writing to file: " + filename );

    // Create and initialise the file.
    var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
    file.initWithPath( filename );

    // To write to the file, make an output stream, initialize the stream with
    // the file, and write to it.
    var outStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance( Components.interfaces.nsIFileOutputStream );

    try
    {
        outStream.init( file.QueryInterface( Components.interfaces.nsIFile ), 0x04 | 0x08 | 0x20, 0664, 0 );  // read-write, create, truncate
        // octal 0664 gives "664" permissions.
        var result = outStream.write( content, content.length );
    }
    catch ( e )
    {
        // Log!
        log( 0, e );
    }
    finally
    {
        outStream.close();
    }

    log( 3, "Write completed." );
}


/**
 * Backup a file by copying the contents and appending ".orig" to the filename.
 *
 * @param filename (String) The full path of the file to backup.
 */
function backupFile( filename )
{
    log( 2, "Backing up " + filename );

    try
	{
        // Create and initialise the file.
        var file = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
        file.initWithPath( filename );

		// Remove any existing backup.
		var backupFile = Components.classes["@mozilla.org/file/local;1"].createInstance( Components.interfaces.nsILocalFile );
        backupFile.initWithPath( filename + ".orig" );
		if (backupFile.exists() == true)
		{
			backupFile.remove(false);
		}

		// Note nsIFile.moveTo() fails on win32.
		file.copyTo( null, file.leafName + ".orig" );
        log( 3, "Backup completed." );
    }
    catch( e )
    {
        log( 1, e );
        log( 3, "Backup failed." );
   }
}


/**
 * Create folders/directories. The folder will have 775 permissions. Does
 * nothing if the folder already exists.
 *
 * @param foldername (String) The full path of the folder to create.
 * @throws Exception if an error occurs.
 */
function createFolders( foldername )
{
    try
    {
        var localFile = Components.classes['@mozilla.org/file/local;1'].
            createInstance( Components.interfaces.nsILocalFile );
        localFile.initWithPath( foldername );

        if ( localFile.exists() )
        {
            log( 2, foldername + " already exists." );
            return;
        }

        localFile.create( localFile.DIRECTORY_TYPE, 0775 );
    }
    catch ( e )
    {
        throw "Caught exception: [configureInstall.createFolders()] " + e;
    }
}


/**
 * Returns the platform-specific end-of-line character sequence.
 */
function getEndOfLineChars()
{
	if( ncipackage.shortArch == "win32" )
	{
		return "\r\n";
	}
	else
	{
		return "\n";
	}
}


/**
 * Log a message.
 *
 * @param level (int) The level to log with. Valid levels are: 0 - Error,
 *        1 - Warning 2 - Info, 3 - Debug.
 * @param msg (String) The message to log.
 */
function log( level, msg )
{
    try
    {
        var logger = Components.classes["@micromuse.com/NCILog;1"].getService( Components.interfaces.nsINCILog);

        switch ( level )
        {
            case 0: logger.error( msg ); break;
            case 1: logger.warning( msg ); break;
            case 2: logger.info( msg ); break;
            case 3: logger.debug( msg ); break;
            default: break;
        }
    }
    catch ( e )
    {
        // ??
    }
}

/**
 * Translate a filename so that it uses the correct path separator based on the
 * local platform.
 *
 * @param filename (String) The supplied filename
 *
 * @return filename (String) The file path updated with the correct path separators.
 */
function transformPathSeparators( filename )
{
	var newContents;
	if( ncipackage.shortArch == "win32" )
	{
		// Replace ObjectServer port.
		regex = /\//g;
		newContents = filename.replace( regex,
			"\\" );
	}
	else
	{
		regex = /\\/g;
		newContents = filename.replace( regex,
			"\/" );
	}
	return newContents;
}

